package tech.nevets.modbot;

import ch.qos.logback.classic.Level;
import net.dv8tion.jda.api.JDA;
import net.dv8tion.jda.api.JDABuilder;
import net.dv8tion.jda.api.hooks.ListenerAdapter;
import net.dv8tion.jda.api.utils.cache.CacheFlag;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import tech.nevets.modbot.api.BotModule;
import tech.nevets.modbot.api.Config;
import tech.nevets.modbot.util.commands.CoreListener;

import javax.security.auth.login.LoginException;
import java.util.*;

public class CoreBot {
    public static JDA jda;
    public static final Config CORE_CONFIG = new Config("core");
    public static final List<ListenerAdapter> PLUGIN_LISTENERS = new ArrayList<>();
    private static String botPrefix;
    private static List<BotModule> modules;
    private static final Logger LOGGER = LoggerFactory.getLogger(CoreBot.class);

    public static void main(String[] args) {
        setLoggingLevel(Level.INFO);
        CORE_CONFIG.addDefaults(getDefaults());
        CORE_CONFIG.loadConfig();
        botPrefix = CORE_CONFIG.getConfig().getString("bot.prefix");
        modules = ModuleLoader.loadModules();
        CoreListener coreListener = new CoreListener();

        JDABuilder builder = JDABuilder.createDefault(CORE_CONFIG.getConfig().getString("bot.token"))
                .enableCache(CacheFlag.VOICE_STATE)
                .addEventListeners(coreListener);

        for (ListenerAdapter listener : PLUGIN_LISTENERS) {
            builder.addEventListeners(listener);
        }

        try {
            jda = builder.build();
        } catch (LoginException e) {
            e.printStackTrace();
        }

        coreListener.getCommandRegistry().registerSlashCommands();
        for (BotModule module : modules) {
            if (module.loadCommandRegistry() == null) {
                LOGGER.warn("Module has no commands to load");
            } else {
                module.loadCommandRegistry().registerSlashCommands();
            }
            module.onEnable(jda);
        }

    }

    public static void addListener(ListenerAdapter listener) {
        PLUGIN_LISTENERS.add(listener);
    }


    private static Map<String, Object> getDefaults() {
        Map<String, Object> defaults = new HashMap<>();
        defaults.put("bot.token", "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx");
        defaults.put("bot.prefix", "!");

        return defaults;
    }

    private static void setLoggingLevel(Level level) {
        ch.qos.logback.classic.Logger root = (ch.qos.logback.classic.Logger) LoggerFactory.getLogger(Logger.ROOT_LOGGER_NAME);
        root.setLevel(level);
    }

    public static String getPrefix() {
        return botPrefix;
    }
}
